@@ -1,16 +1,26 @@
-/* 
+/*
  * Copyright (C) 2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -21,7 +31,6 @@
 #include <algorithm>
 
 #include <cxxtools/loginit.h>
-#include <cxxtools/log.h>
 #include <cxxtools/arg.h>
 
 #include <tntdb/connect.h>
@@ -33,8 +42,6 @@
 
 #include "util.h"
 
-log_define("sqlcmd.main")
-
 int main(int argc, char* argv[])
 {
   try
@@ -45,11 +52,7 @@ int main(int argc, char* argv[])
     cxxtools::Arg<bool> statement(argc, argv, 'p');
     cxxtools::Arg<bool> cursor(argc, argv, 'c');
 
-    cxxtools::Arg<bool> debug(argc, argv, 'd');
-    if (debug)
-      log_init_debug();
-    else
-      log_init();
+    log_init();
 
     if (argc < 2)
     {
@@ -62,7 +65,6 @@ int main(int argc, char* argv[])
                    "  -r        select row\n"
                    "  -p        select with prepared statement\n"
                    "  -c        select with cursor\n"
-                   "  -d        enable debug\n\n"
                    "When no sql-query is passed, sqlcmd reads a sql-query from standard-input.\n"
                 << std::endl;
       return -1;
