@@ -35,7 +35,12 @@
 #include <tntdb/date.h>
 #include <tntdb/time.h>
 #include <tntdb/datetime.h>
+#include <cxxtools/convert.h>
 #include <tntdb/config.h>
+#include <vector>
+#include <list>
+#include <deque>
+#include <set>
 
 namespace tntdb
 {
@@ -85,6 +90,11 @@ namespace tntdb
        */
       Statement& setBool(const std::string& col, bool data)
         { stmt->setBool(col, data); return *this; }
+      /**
+       * Sets the hostvariable with the given name to a short value.
+       */
+      Statement& setShort(const std::string& col, short data)
+        { stmt->setShort(col, data); return *this; }
       /**
        * Sets the hostvariable with the given name to a int value.
        */
@@ -95,13 +105,18 @@ namespace tntdb
        */
       Statement& setLong(const std::string& col, long data)
         { stmt->setLong(col, data); return *this; }
+      /**
+       * Sets the hostvariable with the given name to a unsigned short value.
+       */
+      Statement& setUnsignedShort(const std::string& col, unsigned short data)
+        { stmt->setUnsignedShort(col, data); return *this; }
       /**
        * Sets the hostvariable with the given name to a unsigned value.
        */
       Statement& setUnsigned(const std::string& col, unsigned data)
         { stmt->setUnsigned(col, data); return *this; }
       /**
-       * Sets the hostvariable with the given name to a unsigned value.
+       * Sets the hostvariable with the given name to a unsigned long value.
        */
       Statement& setUnsignedLong(const std::string& col, unsigned long data)
         { stmt->setUnsignedLong(col, data); return *this; }
@@ -191,6 +206,46 @@ namespace tntdb
       Statement& set(const std::string& col, const T& data);
       //@}
 
+      /*
+       * Sets multiple numbered parameters to the values specified by the iterator range.
+       *
+       * The method expects, that the statement has columns with the specified column name
+       * appended by a number range starting from 0. This list can be generated for example
+       * with tntdb::SqlBuilder.
+       *
+       * Example:
+       *
+       * \code
+       *   std::vector<int> values;
+       *   values.push_back(5);
+       *   values.push_back(12);
+       *   tntdb::Statement stmt = conn.prepare("select a, b, c from tab1 where v in (" + tntdb::Statement::paramlist("v", values.size()) + ")");
+       *   stmt.set("v", values.begin(), values.end());
+       *   // or short version:
+       *   stmt.set("v", values);
+       *
+       *   // now stmt is ready for iteration
+       * \endcode
+       */
+      template <typename Iterator>
+      Statement& set(const std::string& col, Iterator it1, Iterator it2);
+
+      /**
+       * Set the hostvariable with the given name to the passed value or null.
+       *
+       * The method sets the hostvariable to the given value if the 2nd
+       * argument is true. Otherwise the hostvariable is set to null.
+       */
+      template <typename T>
+      Statement& setIf(const std::string& col, bool notNull, const T& data)
+      {
+        if (notNull)
+          set(col, data);
+        else
+          setNull(col);
+        return *this;
+      }
+
       /// statement-execution-methods
       /**
        * Executes a query with the current parameters. The query should not
@@ -236,6 +291,7 @@ namespace tntdb
        * Returns the actual implementation-class.
        */
       const IStatement* getImpl() const  { return &*stmt; }
+      IStatement* getImpl()              { return &*stmt; }
   };
 
   /**
@@ -260,11 +316,11 @@ namespace tntdb
         { stmt.setBool(name, data); }
       void setInt(int data)
         { stmt.setInt(name, data); }
-      void setLong(int data)
+      void setLong(long data)
         { stmt.setLong(name, data); }
       void setUnsigned(unsigned data)
         { stmt.setUnsigned(name, data); }
-      void setUnsignedLong(unsigned data)
+      void setUnsignedLong(unsigned long data)
         { stmt.setUnsignedLong(name, data); }
       void setInt32(int32_t data)
         { stmt.setInt32(name, data); }
@@ -360,7 +416,6 @@ namespace tntdb
   }
 #endif
 
-  // TODO
   inline void operator<< (Hostvar& hostvar, const Decimal& data)
   {
     hostvar.setDecimal(data);
@@ -410,6 +465,23 @@ namespace tntdb
   {
     hostvar.setDatetime(data);
   }
+
+  template <typename T>
+  void operator<< (Hostvar& hostvar, const std::vector<T>& data)
+  { hostvar.getStatement().set(hostvar.getName(), data.begin(), data.end()); }
+
+  template <typename T>
+  void operator<< (Hostvar& hostvar, const std::list<T>& data)
+  { hostvar.getStatement().set(hostvar.getName(), data.begin(), data.end()); }
+
+  template <typename T>
+  void operator<< (Hostvar& hostvar, const std::deque<T>& data)
+  { hostvar.getStatement().set(hostvar.getName(), data.begin(), data.end()); }
+
+  template <typename T>
+  void operator<< (Hostvar& hostvar, const std::set<T>& data)
+  { hostvar.getStatement().set(hostvar.getName(), data.begin(), data.end()); }
+
   //@}
 
   template <typename T>
@@ -420,6 +492,16 @@ namespace tntdb
     return *this;
   }
 
+  template <typename Iterator>
+  Statement& Statement::set(const std::string& col, Iterator it1, Iterator it2)
+  {
+    for (unsigned n = 0; it1 != it2; ++n, ++it1)
+    {
+      set(col + cxxtools::convert<std::string>(n), *it1);
+    }
+    return *this;
+  }
+
   template <typename T>
   void Hostvar::set(const T& data)
   {
