@@ -35,6 +35,7 @@
 #include <cxxtools/log.h>
 #include <new>
 #include <sstream>
+#include <poll.h>
 
 log_define("tntdb.postgresql.connection")
 
@@ -43,6 +44,7 @@ namespace tntdb
   namespace postgresql
   {
     Connection::Connection(const char* conninfo)
+      : transactionActive(0)
     {
       log_debug("PQconnectdb(\"" << conninfo << "\")");
 
@@ -52,6 +54,8 @@ namespace tntdb
 
       if (PQstatus(conn) == CONNECTION_BAD )
         throw PgConnError("PQconnectdb", conn);
+
+      log_debug("connected to postgresql backend process " << PQbackendPID(conn));
     }
 
     Connection::~Connection()
@@ -59,6 +63,7 @@ namespace tntdb
       if (conn)
       {
         clearStatementCache();
+        currvalStmt = tntdb::Statement();
 
         log_debug("PQfinish(" << conn << ")");
         PQfinish(conn);
@@ -67,17 +72,21 @@ namespace tntdb
 
     void Connection::beginTransaction()
     {
-      execute("BEGIN");
+      if (transactionActive == 0)
+        execute("BEGIN");
+      ++transactionActive;
     }
 
     void Connection::commitTransaction()
     {
-      execute("COMMIT");
+      if (transactionActive == 0 || --transactionActive == 0)
+        execute("COMMIT");
     }
 
     void Connection::rollbackTransaction()
     {
-      execute("ROLLBACK");
+      if (transactionActive == 0 || --transactionActive == 0)
+        execute("ROLLBACK");
     }
 
     Connection::size_type Connection::execute(const std::string& query)
@@ -146,26 +155,71 @@ namespace tntdb
     bool Connection::ping()
     {
       log_debug("ping()");
-      try
-      {
-        select("select 1");
-        return true;
-      }
-      catch (const PgError&)
+
+      if (PQsendQuery(conn, "select 1") == 0)
       {
+        log_debug("failed to send statement \"select 1\" to database in Connection::ping()");
         return false;
       }
+
+      while (true)
+      {
+        struct pollfd fd;
+        fd.fd = PQsocket(conn);
+        fd.events = POLLIN;
+        log_debug("wait for input on fd " << fd.fd);
+        if (::poll(&fd, 1, 10000) != 1)
+        {
+          log_debug("no data received in Connection::ping()");
+          return false;
+        }
+
+        log_debug("consumeInput");
+        if (PQconsumeInput(conn) == 0)
+        {
+          log_debug("PQconsumeInput failed in Connection::ping()");
+          return false;
+        }
+
+        log_debug("check PQisBusy");
+        while (PQisBusy(conn) == 0)
+        {
+          log_debug("PQgetResult");
+          PGresult* result = PQgetResult(conn);
+
+          log_debug("PQgetResult => " << static_cast<void*>(result));
+          if (result == 0)
+            return true;
+
+          log_debug("PQfree");
+          PQclear(result);
+        }
+      }
     }
 
     long Connection::lastInsertId(const std::string& name)
     {
-      if (!currvalStmt)
-        currvalStmt = prepare("select currval(:name)");
-
       long ret;
-      currvalStmt.set("name", name)
-        .selectValue()
-        .get(ret);
+
+      if (name.empty())
+      {
+        if (!lastvalStmt)
+          lastvalStmt = prepare("select lastval()");
+
+        lastvalStmt
+          .selectValue()
+          .get(ret);
+      }
+      else
+      {
+        if (!currvalStmt)
+          currvalStmt = prepare("select currval(:name)");
+
+        currvalStmt.set("name", name)
+          .selectValue()
+          .get(ret);
+
+      }
 
       return ret;
     }
